@@ -2,6 +2,16 @@ from enum import Enum
 
 __all__ = ["entity","events","messages","templates"]
 
+_dust_logger = None
+
+def set_logger(logger):
+    global _dust_logger
+    _dust_logger = logger
+
+def logger():
+    global _dust_logger
+    return _dust_logger
+
 class Datatypes(Enum):
     INT = 0
     NUMERIC = 2
@@ -55,6 +65,14 @@ class MetaProps(Enum):
     @property
     def id_value(self):
         return self.value[3]
+    
+    def to_json(self):
+        return {
+            "datatype": self.datatype.name,
+            "valuetype": self.valuetype.name,
+            "order_value": self.order_value,
+            "id_value": self.id_value
+        }
 
 class FieldProps(Enum):
     @property
