@@ -48,6 +48,32 @@ class EntityTypes(FieldProps):
     unit = (UNIT_ENTITY_META, UnitMeta, 3)
     meta_field = (UNIT_ENTITY_META, MetaField, 4)
 
+class EntityJsonSerializer():
+    def dump(self, obj, fp):
+        if isinstance(obj, Entity):
+            fp.write(json.dumps(obj.to_json()).encode())
+            fp.write(b"\n")
+        elif isinstance(obj, list):
+            objs = []
+            for e in obj:
+                if isinstance(e, Entity):
+                    objs.append(e.to_json())
+                else:
+                    raise ValueError("Only entity object can be serialized!")
+            fp.write(json.dumps(objs).encode())
+            fp.write(b"\n")
+        else:
+            raise ValueError("Only entity object can be serialized!")
+
+    def load(self, f):
+        try:
+            data = json.loads(f.readline().decode())
+            if not isinstance(data, list):
+                data = [data]
+            return Store.from_json(data)
+        except:
+            raise ValueError("Invalid json data in messagequeue!")
+
 def get_unit_deps_tuple(module_name, unit_name, meta_type_enums):
     module = import_module(module_name)
     unit_name_attr = getattr(module, unit_name)
@@ -55,9 +81,13 @@ def get_unit_deps_tuple(module_name, unit_name, meta_type_enums):
     dep_func = getattr(module, "get_unit_dependencies", None)
     return (unit_name_attr, meta_type_enums_attr, dep_func)
 
-def compare_entity_to_json_simple(meta_type_enum, entity, json_entity, json_entity_map, compare_sub_entity_fields, log_prefix=""):
+def compare_entity_to_json_simple(meta_type_enum, entity, json_entity, json_entity_map, compare_sub_entity_fields_map=None, log_prefix=""):
     changed = {}
 
+    compare_sub_entity_fields = {}
+    if compare_sub_entity_fields_map:
+        compare_sub_entity_fields = compare_sub_entity_fields_map.get(meta_type_enum, {})
+
     for field in meta_type_enum.fields_enum:
         if field.valuetype == ValueTypes.SINGLE:
             if field.datatype == Datatypes.ENTITY:
@@ -72,10 +102,33 @@ def compare_entity_to_json_simple(meta_type_enum, entity, json_entity, json_enti
                     if orig_value is None and not new_value is None or not orig_value is None and new_value is None:
                         changed[log_prefix+field.name] = {"orig_value": orig_value, "new_value": new_value}
                     elif not orig_value is None and not new_value is None:
-                        changed.update(compare_entity_to_json_simple(compare_sub_entity_fields[field], orig_value, new_value, json_entity_map, None, log_prefix="{}#".format(field.name)))
+                        changed.update(compare_entity_to_json_simple(compare_sub_entity_fields[field], orig_value, new_value, json_entity_map, compare_sub_entity_fields_map, log_prefix="{}#".format(field.name)))
             else:
-                if ( json_entity is None and not entity is None ) or ( not json_entity is None and entity is None ) or \
-                     json_entity and entity.access(Operation.GET, None, field) != json_entity.get(Store.get_global_fieldname(field)):
+                entity_changed = False
+                if ( json_entity is None and not entity is None ) or ( not json_entity is None and entity is None ):
+                    entity_changed = True
+                elif json_entity and entity:
+                    value1 = entity.access(Operation.GET, None, field)
+                    value2 = json_entity.get(Store.get_global_fieldname(field))
+                    if value1 is None and not value2 is None or not value1 is None and value2 is None:
+                        entity_changed = True
+                    elif value1 and value2:
+                        if field.datatype == Datatypes.BOOL:
+                            entity_changed = bool(value1) != bool(value2)
+                        elif field.datatype == Datatypes.INT:
+                            entity_changed = int(value1) != int(value2)
+                        elif field.datatype == Datatypes.NUMERIC:
+                            entity_changed = abs(float(value1) - float(value2)) > 0.0001
+                        elif field.datatype == Datatypes.STRING:
+                            entity_changed = str(value1) != str(value2)
+                        elif field.datatype == Datatypes.JSON:
+                            dd = deepdiff.DeepDiff(value1, value2, ignore_order=True)
+                            if dd:
+                                entity_changed = True
+                        else:
+                            entity_changed = value1 != value2
+                    
+                if entity_changed:
                     changed[log_prefix+field.name] = {"orig_value": entity.access(Operation.GET, None, field), "new_value": json_entity.get(Store.get_global_fieldname(field))}
 
         elif field.valuetype == ValueTypes.SET or field.valuetype == ValueTypes.LIST or field.valuetype == ValueTypes.MAP:
@@ -117,7 +170,7 @@ def compare_entity_to_json_simple(meta_type_enum, entity, json_entity, json_enti
                                 for new_entity_global_id in match_to:
                                     new_value = json_entity_map[new_entity_global_id]
                                     if not new_value is None:
-                                        sub_changed = compare_entity_to_json_simple(compare_sub_entity_fields[field], orig_value, new_value, json_entity_map, None, log_prefix="{}#".format(field.name))
+                                        sub_changed = compare_entity_to_json_simple(compare_sub_entity_fields[field], orig_value, new_value, json_entity_map, compare_sub_entity_fields_map, log_prefix="{}#".format(field.name))
                                         if len(sub_changed) == 0:
                                             match_found = True
                                             break
@@ -222,6 +275,7 @@ class Store():
 
         return e
 
+    @staticmethod
     def _add_entity_to_store(e):
         entity_map = globals()["_entity_map"]
 
@@ -234,6 +288,12 @@ class Store():
             committed_field: e.committed.name
         }, e)
 
+    @staticmethod
+    def _delete_entity_to_store(e):
+        entity_map = globals()["_entity_map"]
+
+        if e.global_id() in entity_map:
+            del entity_map[e.global_id()]
 
     @staticmethod
     def increment_unit_counter(unit, requested_id):
@@ -252,8 +312,6 @@ class Store():
 
     @staticmethod
     def load_types_from_enum(e, unit_meta_id):
-        entity_map = globals()["_entity_map"]
-        #meta_ref = globals()["_meta_ref"]
         enum_map = globals()["_enum_map"]
         unit_meta_types = {}
         for meta_type in e:
@@ -264,7 +322,7 @@ class Store():
 
         for base_field in EntityBaseMeta:
             global_name = Store._global_field_name(UNIT_ENTITY_META, EntityTypes._entity_base.name, base_field.name)
-            enum_map[global_name] = {"datatype": base_field.value[0], "valuetype": base_field.value[1], "id": field.id_value, "field_order": base_field.order_value, "_enum": base_field}
+            enum_map[global_name] = {"datatype": base_field.value[0], "valuetype": base_field.value[1], "id": base_field.id_value, "field_order": base_field.order_value, "_enum": base_field}
             enum_map[base_field] = global_name
 
         for meta_type in e:
@@ -282,12 +340,6 @@ class Store():
                 enum_map[field_meta_type] = meta_type
                 enum_map[field_meta_type.global_id()] = meta_type
 
-                #print("Unit name: {} ({}), meta_type: {}, global id: {}".format(unit.access(Operation.GET, None, UnitMeta.name), field_meta_type.unit.global_id(), meta_type.name ,field_meta_type.global_id()))
-
-
-            field_config = {}
-
-            max_order = 0
             field_entities = []
             max_id_value = 0
             for field in meta_type.fields_enum:
@@ -304,7 +356,6 @@ class Store():
                 if field.id_value > max_id_value:
                     max_id_value = field.id_value
 
-
             for field_entity in field_entities:
                 field_meta_type.access(Operation.ADD, field_entity, TypeMeta.fields)
 
@@ -336,6 +387,9 @@ class Store():
         last_global_id = None
         last_entity_path = []
 
+        unit = None
+        meta_type = None
+
         if len(path) > 0:
             if operation == Operation.WALK and isinstance(path[0], list):
                 last_obj = path[0]
@@ -347,7 +401,9 @@ class Store():
                     if len(path) > 2 and ( path[1] is None or isinstance(path[1], int) ) and \
                         ( isinstance( path[2], FieldProps ) or isinstance( path[2], Entity ) ):
                         entity_id = Store.increment_unit_counter(path[0], path[1])
-                        local_ref = Entity._ref(path[0], entity_id, path[2])
+                        unit = path[0]
+                        meta_type = path[2]
+                        local_ref = Entity._ref(unit, entity_id, meta_type)
                         idx = 3
                 else:
                     # found entity id on path[0]
@@ -362,7 +418,7 @@ class Store():
         #print("Access 1: local_ref={}, path={}".format(local_ref, path))
         if local_ref:
             if not local_ref in entities:
-                last_obj = Store._create_entity(path[0], entity_id, path[2])
+                last_obj = Store._create_entity(unit, entity_id, meta_type)
             else:
                 last_obj = entities[local_ref][1]
             last_global_id = last_obj.global_id()
@@ -479,7 +535,8 @@ class Store():
                 e_map[global_field_name] = set()
             elif field_config["valuetype"] == ValueTypes.LIST:
                 e_map[global_field_name] = []
-                return e_map[global_name]
+            
+            return e_map[global_field_name]
 
     @staticmethod
     def _set_uncommitted(e, e_map):
@@ -513,7 +570,7 @@ class Store():
                         e_map.setdefault(global_field_name, set()).add(value.global_id())
                         changed = Store._set_uncommitted(entity, e_map)
                 else:
-                    if e_map.get(global_field_name, None) != value:
+                    if e_map.get(global_field_name, None) != value: # TODO: change if it shoudl be value.global_id()
                         e_map[global_field_name] = value.global_id()
                         changed = Store._set_uncommitted(entity, e_map)
             else:
@@ -556,18 +613,18 @@ class Store():
         entities = globals()["_entity_map"]
         enum_map = globals()["_enum_map"]
 
-
         if isinstance(obj, Entity):
             global_name = enum_map[key]
             e_map = entities[obj.global_id()][0]
             if global_name in e_map:
                 unit, entity_id, meta_type = Entity._resolve_global_id(e_map[global_name])
                 if entity_id:
+                    if not e_map[global_name] in entities:
+                        e = Store.access(Operation.GET, None, e_map[global_name])
+                        #print(str(e.global_id()))
                     return entities[e_map[global_name]][1]
                 else:
                     return e_map[global_name]
-
-
         elif isinstance(obj, list):
             if isinstance(key, int) and key < len(obj):
                 return obj[key]
@@ -703,6 +760,14 @@ class Entity():
                 if unit and isinstance(unit, Entity) and enum_map[unit.meta_type] == EntityTypes.unit:
                     if entity and entity.meta_type and isinstance(entity.meta_type, Entity) and entity.meta_type in enum_map:
                         return (unit, entity_id, entity.meta_type)
+                    elif entity is None:
+                        unit_meta = enum_map[unit.access(Operation.GET, None, UnitMeta.name)+"_meta"]
+                        for meta_type_global_id in unit_meta.access(Operation.GET, None, UnitMeta.meta_types):
+                            meta_type_entity = Store.access(Operation.GET, None, meta_type_global_id)
+                            if meta_type_entity.access(Operation.GET, None, TypeMeta.name) == parts[2]:
+                                #print("None existant entity, but create it for {}. Type: {}".format(value, meta_type_entity.access(Operation.GET, None, TypeMeta.name)))
+                                return (unit, entity_id, meta_type_entity)
+
 
         return (None, None, None)
 
@@ -782,9 +847,12 @@ class Entity():
                     e.access(Operation.SET, value, field_config["_enum"])
             elif field == committed_field:
                 if Committed[value] == Committed.SAVED:
-                    self.set_committed()
+                    e.set_committed()
         return e
 
+    def delete(self):
+        Store._delete_entity_to_store(self)
+
 _entity_map = {}
 _enum_map = {}
 
